@@ -30,7 +30,6 @@ plt.style.use(['science', 'notebook', 'grid'])
 
 
 class xydata():
-
     # initiate 2-dimensional xydata
     def __init__(self, rawdatafile, quantity=None, fig=None, title=None, legend =None):
         """
@@ -53,7 +52,7 @@ class xydata():
         self.quantity = ['x', 'a.u.', 'y', 'a.u.'] if quantity is None else quantity 
         self.legend = rawdatafile if legend is None else legend
         self.title = rawdatafile if title is None else title
-        self.fig = [20, '#e6f0f8'] if fig is None else fig
+        self.fig = [18, '#e6f0f8'] if fig is None else fig
         self.xlabel = self.quantity[0] + ' (' + self.quantity[1] + ')'
         self.ylabel = self.quantity[2] + ' (' + self.quantity[3] + ')' 
         
@@ -71,8 +70,8 @@ class xydata():
         self.tmpfitpeaks = []
         self.tmpfitpara = []
         self.manu_peaks = pd.DataFrame(columns=['code', 'label', 'cen', 'wid', 'amp'])
-        #reference peaks from each subclass
-        self.refpeaks = pd.DataFrame(columns=['code', 'Materials', 'label', 'cen'])
+        #reference peaks
+        self.refpeaks = pd.DataFrame(columns=['code', 'label', 'cen'])
 
  
         
@@ -361,23 +360,33 @@ class xydata():
         peak is the output of fit_lorentz function
         {"label", "cen", "amp", "xfit", "yfit", "pcov"}
         """
-        fig = plt.figure(figsize=(self.fig[0],aspect*self.fig[0]), constrained_layout=True)
-        gs = fig.add_gridspec(3, 3);
-        ax1 = fig.add_subplot(gs[0:2, 0:2])
-        ax2 = fig.add_subplot(gs[0:2, 2])
-        ax3 = fig.add_subplot(gs[2, 0:3])
+        xmin = peak['cen']-peak['wid']
+        xmax = peak['cen']+peak['wid']
+        ref = self.refpeaks[(self.refpeaks['cen']>xmin) & (self.refpeaks['cen']<xmax)]
+        num_refs = len(ref.index)
+        fig_w = self.fig[0]
+        fig_h = 0.3*self.fig[0]+(num_refs+1)*0.5
+        fig_f = 0.15*self.fig[0]
+        fig_l = (num_refs+1)*0.5
         
-        fig.suptitle(peak['label'] + ' fitting')
+        fig, (ax0, ax1, ax2) = plt.subplots(3, 1, figsize=(fig_w,fig_h), gridspec_kw={'height_ratios': [fig_f, fig_f, fig_l]}, constrained_layout=True)
+        #plt.tight_layout()
         
+        # plot the full range and add a vertical line in the full range data for the peak
+        self.plot_peaks(ax=ax0, xrange=[min(self.data['x']), max(self.data['x'])])
+        ylim = ax0.get_ylim()
+        ax0.vlines(x=peak['cen'], ymin = ylim[0], ymax = ylim[1], color='red')
+        
+        # Plot the fitting with raw data and reference line
         ax1.set_facecolor(self.fig[1])
         ax1.set_xlabel(self.xlabel, fontsize = 16)
         ax1.set_ylabel(self.ylabel, fontsize = 16)          
         
-        dfit = self.data[(self.data['x']>peak['cen']-peak['wid']) & (self.data['x']<peak['cen']+peak['wid'])]
-        ax1.scatter(dfit.x, dfit.y)
-        ax1.errorbar(dfit.x, dfit.y, dfit.err, ls='', color = 'k')
-        ax1.plot(peak['xfit'], peak['yfit'], color='r')
-        
+        dfit = self.data[(self.data['x']>xmin) & (self.data['x']<xmax)] # select raw data
+        ax1.scatter(dfit.x, dfit.y) # raw data
+        ax1.errorbar(dfit.x, dfit.y, dfit.err, ls='', color = 'k') # errors
+        ax1.plot(peak['xfit'], peak['yfit'], color='r') # fitting
+
         ymin, ymax = ax1.get_ylim()
         ymax = ymax+0.1*(ymax-ymin)
         ax1.axvline(peak['cen'], ymin=0, ymax = (peak['amp']-ymin)/(ymax-ymin), color='grey', linestyle='--')
@@ -385,73 +394,96 @@ class xydata():
         ax1.annotate(peak['label']+": "+"{:.2f}".format(peak['cen']), xy=(peak['cen'], peak['amp']), xycoords='data', size=16, ha='center', va='bottom', color = 'blue')
         
         # Evaluate the errors in fitting
-        im2 = ax2.imshow(np.log(np.abs(peak['pcov'])))
-        fig.colorbar(im2, ax = ax2)
-        self.plot_peaks(ax=ax3, xrange=[min(self.data['x']), max(self.data['x'])])
-        # add a vertical line in the full range data for the peak
-        ylim = ax3.get_ylim()
-        ax3.vlines(x=peak['cen'], ymin = ylim[0], ymax = ylim[1], color='red')
+        rect = [0.8,0,0.2,1]
+        ax1.set_xlim([xmin, xmax + 0.25*(xmax-xmin)])
+        ax1_inset = ax1.inset_axes([0.85,0,0.2,1])
+        ax1_inset.axis('off')
+        im2 = ax1_inset.imshow(np.log(np.abs(peak['pcov'])))
+        #fig.colorbar(im2, ax = ax1_inset)
 
+        # Add reference peaks in the range of the selected peak
+        if num_refs >0:
+            ref_draw = ref.drop(['label'], axis=1)
+            ref_draw.insert(loc=0, column = 'label', value=[str(i) for i in np.arange(len(ref))])
+            ref.insert(loc=0, column = 'index', value= [str(i) for i in np.arange(len(ref))])
+            xydata.add_vlines(ax1, [xmin, xmax], ref_draw, frompeak=False, linecolor ='blue') # add the reference line
+            bbox=[0, 0, 1, 1]
+            font_size = 14
+            ax2.axis('off')
+            ax2.set_title('Reference peaks from literature', loc ='left')
+            table = ax2.table(cellText = ref.values, bbox=bbox, colLabels=ref.columns, rowLoc='center') #rowLabels=self.tmp_peaks.index,
+            table.auto_set_font_size(False)
+            table.set_fontsize(font_size)
+        else:
+            ax2.axis('off')
         plt.show()
     
     # analyze identified peaks
-    def analyse_peaks(self, peak):
+    def analyse_peaks(self, peak_selected):
         if len(self.peaks.index) == 0:
             print("No peak has been identified.")
             return False
-        i, label, c = peak.split(' ')
-        cen = float(c)
-        index = int(i)
-        peak = self.peaks[self.peaks['cen']==cen]
-        self.tmp_peakfit = self.fit_lorentz(peak.iloc[0].to_dict())
-        if self.tmp_peakfit is None:
-            print('peak fitting failed.')
+        
+        def unpack_options(peak):
+            code, label, cen = peak.split(' ')
+            cen = float(cen)
+            code = int(code)
+            return code, label, cen
+        
+        # Select the peak 
+        code, label, cen = unpack_options(peak_selected)
+        peak = self.peaks[self.peaks['code']==code]
+        
+        if len(peak.index)>0:
+            self.tmp_peakfit = self.fit_lorentz(peak.iloc[0].to_dict())
+            if self.tmp_peakfit is None:
+                print('peak fitting failed.')
+            else:
+                self.plot_fitting(self.tmp_peakfit)
         else:
-            self.plot_fitting(self.tmp_peakfit)
+            print('No peak selected.')
+        
     
     # process peaks with widgets
     def process_peaks(self):
         if len(self.peaks.index) == 0:
             print("No peak has been identified.")
             return False
-        print('Process the peaks.')
-        def get_options():
+        
+        # Format the labels for the peaks
+        def pack_options(peaks):
             try:
-                self.peaks = self.peaks.reset_index(drop=True)  # make sure indexes pair with number of rows
-                self.peaks['code'] = np.arange(len(self.peaks))
+                peaks = peaks.reset_index(drop=True)  # make sure indexes pair with number of rows
+                peaks['code'] = np.arange(len(peaks))
             except:
-                pass
+                print('I cannot pack options for the widgets.')
+                return False
             options = []
             labels = []
-            for index, row in self.peaks.iterrows():
+            for index, row in peaks.iterrows():
                 options.append('{} {} {}'.format(row['code'],row['label'],row['cen']))
                 labels.append(row['label'])
             return options, labels
-
-        options, labels = get_options()
-
+        
+        # Peaks found from the raw data
+        options, labels = pack_options(self.peaks)
+        lb_peaks = widgets.HTML(value="<b>Choose a peak:</b>")
         dp_peaks = widgets.Dropdown(placeholder='choose a peak',options=options, value=options[0],
                                     description='select a peak:',
                                     disabled=False, continuous_update=True, orientation='horizontal',readout=True, readout_format='.1f',
-                                    style = {'description_width': 'initial'}, layout=widgets.Layout(width='80%', height='100%'))
+                                    style = {'description_width': 'initial'}, layout=widgets.Layout(width='80%', height='80%'))
         txt_label = widgets.Text(value=labels[0], placeholder='enter a label for the peak', description='peak label:',
                                  disabled=False, continuous_update=True, orientation='horizontal',readout=True, readout_format='.1f',
-                                 style = {'description_width': 'initial'}, layout=widgets.Layout(width='80%', height='100%'))
-
-        #buttons for taking the data
+                                 style = {'description_width': 'initial'}, layout=widgets.Layout(width='80%', height='80%'))
+        
+        # buttons for taking the data
         btn_delete_peak = widgets.Button(description='delete peak', disabled=False, button_style='', tooltip='Click me', layout=widgets.Layout(width='80%', height='80%'))
         btn_update_peak = widgets.Button(description='update peak', disabled=False, button_style='', tooltip='Click me', layout=widgets.Layout(width='80%', height='80%'))
 
-        controls = widgets.interactive(self.analyse_peaks, peak=dp_peaks);
+        controls = widgets.interactive(self.analyse_peaks, peak_selected=dp_peaks);
         output = controls.children[-1]
-        grid =  widgets.GridspecLayout(3, 5, width=str(self.fig[0]*0.65)+'in')
-        grid[0, :2] = dp_peaks
-        grid[0, 2:4] = txt_label
-        grid[0, 4] = btn_update_peak
-        grid[1, 4] = btn_delete_peak
-        display(grid)
-        display(widgets.VBox([output]))
 
+        # Button controls
         def on_btn_delete_peak(b):
             if len(self.peaks.index) == 1:
                 print("Only 1 peak left. You cannot delete it.")
@@ -466,7 +498,7 @@ class xydata():
                     self.peaks['code'] = np.arange(len(self.peaks))
                 except:
                     pass
-                options, labels = get_options()
+                options, labels = pack_options(self.peaks)
                 dp_peaks.options = options
 
         btn_delete_peak.on_click(on_btn_delete_peak)
@@ -485,12 +517,25 @@ class xydata():
                 #self.peaks.loc[int(i), 'yfit']=self.tmp_peakfit['yfit']
                 #self.peaks.loc[int(i), 'pcov']=self.tmp_peakfit['pcov']
                 print("peak "+dp_peaks.value+ " has been updated.")
-                options, labels = get_options()
+                options, labels = pack_options(self.peaks)
                 dp_peaks.options = options
 
         btn_update_peak.on_click(on_btn_update_peak)
-    
-    #Corrent curve with the reference peak
+        
+        # Plot the controls
+        grid =  widgets.GridspecLayout(2, 6, width=str(self.fig[0]*0.65)+'in')
+        grid[0, :] = lb_peaks
+        grid[1, :2] = dp_peaks
+        grid[1, 2:4] = txt_label
+        grid[1, 4] = btn_update_peak
+        grid[1, 5] = btn_delete_peak
+        
+        display(grid)
+        display(widgets.VBox([output]))
+
+        # display(HTML(refpeak.to_html())) 
+        
+    #Correct curve with the reference peak
     def correct_x(self, refpeak, fitted):
         """
         refpeak: the reference peak, dictionary
@@ -561,7 +606,7 @@ class xydata():
         xydata.add_vlines(ax, [xmin, xmax], peaks)
     
     @staticmethod
-    def add_vlines(ax, xrange, peaks):
+    def add_vlines(ax, xrange, peaks, frompeak=True, linecolor='grey'):
         [ymin, ymax] = ax.get_ylim()
         
         vlines = peaks #
@@ -572,9 +617,9 @@ class xydata():
         except:
             pass
         for index, row in vl.iterrows():
-            vp = (row['amp']-ymin)/(ymax-ymin)
-            ax.axvline(row['cen'], ymin = 0, ymax = vp, color='grey', linestyle='--') 
-            ax.annotate(row['label'], xy=(row['cen'], row['amp']), xycoords='data', size=16, ha='center', va='bottom', color = 'black')
+            vp = (row['amp']-ymin)/(ymax-ymin) if frompeak is True else 1
+            ax.axvline(row['cen'], ymin = 0, ymax = vp, color=linecolor, linestyle='--') 
+            ax.annotate('$'+row['label']+'$', xy=(row['cen'], row['amp'] if frompeak is True else ymax), xycoords='data', size=16, ha='center', va='bottom', color = 'black')
     
     @staticmethod
     def multiple_xy_stack (data, xrange = None, title=None, vlines = None, shiftfactor = 1.01, aspect=0.2, legendloc = 'upper right'):
@@ -615,4 +660,4 @@ class xydata():
         ax.title.set_text(data[0].title if title is None else title)
         
         plt.legend(loc=legendloc, fancybox=True, framealpha=0.5)  #ncol= len(Spectra)
-    
+        
